@@ -17,7 +17,8 @@ namespace Tomboy
 		TomboyTray tray;
 		TomboyGConfXKeybinder keybinder;
 
-		BonoboUIVerb [] menu_verbs;
+		// Keep referenced so our callbacks don't get reaped.
+		static BonoboUIVerb [] menu_verbs;
 
 		public TomboyApplet (IntPtr raw)
 			: base (raw)
@@ -48,15 +49,14 @@ namespace Tomboy
 			OnChangeSize (Size);
 			ShowAll ();
 
-			// Keep referenced so our callbacks don't get reaped.
-			menu_verbs = new BonoboUIVerb [] {
-				new BonoboUIVerb (
-					"Props", new ContextMenuItemCallback (ShowPreferencesVerb)),
-				new BonoboUIVerb (
-					"Plugins", new ContextMenuItemCallback (ShowPluginsVerb)),
-				new BonoboUIVerb (
-					"About", new ContextMenuItemCallback (ShowAboutVerb))
-			};
+			if (menu_verbs == null) {
+				menu_verbs = new BonoboUIVerb [] {
+					new BonoboUIVerb ("Plugins", ShowPluginsVerb),
+					new BonoboUIVerb ("Props", ShowPreferencesVerb),
+					new BonoboUIVerb ("Help", ShowHelpVerb),
+					new BonoboUIVerb ("About", ShowAboutVerb)
+				};
+			}
 
 			SetupMenuFromResource (null, "GNOME_TomboyApplet.xml", menu_verbs);
 		}
@@ -89,9 +89,14 @@ namespace Tomboy
 			manager.PluginManager.ShowPluginsDirectory ();
 		}
 
+		void ShowHelpVerb ()
+		{
+			Tomboy.ActionManager ["ShowHelpAction"].Activate ();
+		}
+
 		void ShowAboutVerb ()
 		{
-			tray.ShowAbout ();
+			Tomboy.ActionManager ["ShowAboutAction"].Activate ();
 		}
 
 		protected override void OnChangeBackground (PanelAppletBackgroundType type, 
@@ -177,18 +182,25 @@ namespace Tomboy
 
 			Gtk.ImageMenuItem item;
 
-			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Preferences..."));
-			item.Image = new Gtk.Image (Gtk.Stock.Properties, Gtk.IconSize.Menu);
-			item.Activated += ShowPreferences;
-			menu.Append (item);
-
 			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Open Plugins Folder"));
 			item.Image = new Gtk.Image (Gtk.Stock.Execute, Gtk.IconSize.Menu);
 			item.Activated += ShowPlugins;
 			menu.Append (item);
 
+			menu.Append (new Gtk.SeparatorMenuItem ());
+
+			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Preferences"));
+			item.Image = new Gtk.Image (Gtk.Stock.Preferences, Gtk.IconSize.Menu);
+			item.Activated += ShowPreferences;
+			menu.Append (item);
+
+			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Help"));
+			item.Image = new Gtk.Image (Gtk.Stock.Help, Gtk.IconSize.Menu);
+			item.Activated += ShowHelpContents;
+			menu.Append (item);
+
 			item = new Gtk.ImageMenuItem (Catalog.GetString ("_About Tomboy"));
-			item.Image = new Gtk.Image (Gnome.Stock.About, Gtk.IconSize.Menu);
+			item.Image = new Gtk.Image (Gtk.Stock.About, Gtk.IconSize.Menu);
 			item.Activated += ShowAbout;
 			menu.Append (item);
 
@@ -205,23 +217,34 @@ namespace Tomboy
 
 		void ShowPreferences (object sender, EventArgs args)
 		{
-			tray.ShowPreferences ();
+			Tomboy.ActionManager ["ShowPreferencesAction"].Activate ();
 		}
 
 		void ShowPlugins (object sender, EventArgs args)
 		{
+			// FIXME: Make this a global action
 			manager.PluginManager.ShowPluginsDirectory ();
 		}
 
+		void ShowHelpContents (object sender, EventArgs args)
+		{
+			Tomboy.ActionManager ["ShowHelpAction"].Activate ();
+		}
+
 		void ShowAbout (object sender, EventArgs args)
 		{
-			tray.ShowAbout ();
+			Tomboy.ActionManager ["ShowAboutAction"].Activate ();
 		}
 
 		void Quit (object sender, EventArgs args)
 		{
-			Logger.Log ("Quitting Tomboy.  Ciao!");
-			Tomboy.Exit (0);
+			Tomboy.ActionManager ["QuitTomboyAction"].Activate ();
+		}
+		
+		public TomboyTray TomboyTray
+		{
+			get { return tray; }
 		}
 	}
 }
+
